<?php

namespace Bazinga\ExposeTranslationBundle\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Output\Output;

/**
 * Command that places translation assets into a given directory.
 *
 * @author Adrien Russo <adrien.russo.qc@gmail.com>
 */
class DumpCommand extends ContainerAwareCommand
{
    /**
     * @see Command
     */
    protected function configure()
    {
        $this
            ->setName('bazinga:expose-translation:dump')
            ->setDefinition(array(
                new InputArgument('target', InputArgument::OPTIONAL, 'The target directory', 'web'),
            ))
            ->addOption('symlink', null, InputOption::VALUE_NONE, 'Symlinks the translation files instead of copying it')
            ->setDescription('Dumps all translation files into a given directory.');
    }

    /**
     * @see Command
     *
     * @throws \InvalidArgumentException When the target directory does not exist
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $target = rtrim($input->getArgument('target'), '/');

        if (!is_dir($target)) {
            throw new \InvalidArgumentException(
                sprintf('The target directory "%s" does not exist.', $input->getArgument('target'))
            );
        }

        if (!function_exists('symlink') && $input->getOption('symlink')) {
            throw new \InvalidArgumentException(
                'The symlink() function is not available on your system. You need to install the assets without the --symlink option.'
            );
        }

        $output->writeln(sprintf(
            'Installing translation files in <comment>%s</comment> directory using the <comment>%s</comment> option',
            $target,
            $input->getOption('symlink') ? 'symlink' : 'hard copy')
        );

        $this
            ->getContainer()
            ->get('bazinga.exposetranslation.dumper.translation_dumper')
            ->dump($target, $input->getOption('symlink'));
    }
}
